@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
 """Array model
 
+.. moduleauthor:: Konrad Hinsen
+
 An array model stores all data in NumPy arrays, in particular the tree
 structure defining the fragments in universes. The array model is an
 in-memory version of the HDF5 storage layout. It is used for HDF5 I/O,
@@ -8,6 +10,13 @@ but can also be of use for interfacing with C or Fortran code.
 
 """
 
+#-----------------------------------------------------------------------------
+#       Copyright (C) 2013 The Mosaic Development Team
+#
+#  Distributed under the terms of the BSD License.  The full license is in
+#  the file LICENSE.txt, distributed as part of this software.
+#-----------------------------------------------------------------------------
+
 import operator
 
 import numpy as N
@@ -28,8 +37,8 @@ fragment_array_dtype = N.dtype([('parent_index', N.uint32),
 
 atom_array_dtype = N.dtype([('parent_index', N.uint32),
                             ('label_symbol_index', N.uint32),
-                            ('dtype_symbol_index', N.uint32),
-                            ('dname_symbol_index', N.uint32),
+                            ('type_symbol_index', N.uint32),
+                            ('name_symbol_index', N.uint32),
                             ('number_of_sites', N.uint32)])
 
 bond_array_dtype = N.dtype([('atom_index_1', N.uint32),
@@ -66,7 +75,6 @@ class Universe(api.MosaicUniverse):
         nmols = len(mol_fragments)
 
         symbol_id = SymbolDict()
-        atom_id = {}
 
         self._atom_array = N.empty((natoms,), atom_array_dtype)
         self._fragment_array = N.empty((nfragments + 1,),
@@ -75,7 +83,8 @@ class Universe(api.MosaicUniverse):
         self._bond_array = N.empty((nbonds,), bond_array_dtype)
         self._molecule_array = N.empty((nmols,), molecule_array_dtype)
 
-        def store_fragment(f, iparent, ifrag, iatom, ibond, isite):
+        def store_fragment(f, iparent, ifrag, iatom, ibond, isite,
+                           path, atom_index):
             self._fragment_array[ifrag]['parent_index'] = iparent
             self._fragment_array[ifrag]['label_symbol_index'] = \
                                                          symbol_id[f.label]
@@ -85,24 +94,24 @@ class Universe(api.MosaicUniverse):
             ifrag += 1
             for ff in f.fragments:
                 ifrag, iatom, ibond, isite = \
-                       store_fragment(ff, iparent, ifrag, iatom, ibond, isite)
+                       store_fragment(ff, iparent, ifrag, iatom, ibond, isite,
+                                      '.'.join([path, ff.label]), atom_index)
             for a in f.atoms:
-                assert len(atom_id) == iatom
-                atom_id[a.unique_id] = iatom
+                atom_index['.'.join([path, a.label])] = iatom
                 self._atom_array[iatom]['parent_index'] = iparent
                 self._atom_array[iatom]['label_symbol_index'] = \
                                                 symbol_id[a.label]
-                self._atom_array[iatom]['dtype_symbol_index'] = \
+                self._atom_array[iatom]['type_symbol_index'] = \
                                                 symbol_id[a.type]
-                self._atom_array[iatom]['dname_symbol_index'] = \
+                self._atom_array[iatom]['name_symbol_index'] = \
                                                 symbol_id[a.name]
                 self._atom_array[iatom]['number_of_sites'] = a.number_of_sites
                 iatom += 1
                 isite += a.number_of_sites
             bonds = []
             for a1, a2, order in f.bonds:
-                i1 = atom_id[f[a1].unique_id]
-                i2 = atom_id[f[a2].unique_id]
+                i1 = atom_index['.'.join([path, a1])]
+                i2 = atom_index['.'.join([path, a2])]
                 if i1 > i2:
                     i1, i2 = i2, i1
                 bonds.append((i1, i2, symbol_id[order]))
@@ -132,7 +141,8 @@ class Universe(api.MosaicUniverse):
             if fragment.is_polymer:
                 polymers.append((ifrag, symbol_id[fragment.polymer_type]))
             ifrag, iatom, ibond, isite = \
-                   store_fragment(fragment, 0, ifrag, iatom, ibond, isite)
+                   store_fragment(fragment, 0, ifrag, iatom, ibond, isite,
+                                  '', {})
             self._molecule_array[imol]['number_of_atoms'] = \
                        iatom - self._molecule_array[imol]['first_atom_index']
             self._molecule_array[imol]['number_of_bonds'] = \
@@ -238,6 +248,33 @@ class Universe(api.MosaicUniverse):
     def number_of_template_sites(self):
         return int(N.sum(self._molecule_array['number_of_sites']))
 
+    # More efficient implementation of various methods
+
+    def recursive_atom_iterator(self):
+        for mol in self._molecule_array:
+            first = mol['first_atom_index']
+            natoms = mol['number_of_atoms']
+            for _ in range(mol['number_of_copies']):
+                for iatom in range(first, first+natoms):
+                    yield Atom(self, iatom)
+
+    def bond_index_array(self):
+        atoms = 0
+        fatoms = 0
+        bonds = []
+        for mol in self._molecule_array:
+            first = mol['first_bond_index']
+            nbonds = mol['number_of_bonds']
+            natoms = mol['number_of_atoms']
+            a1 = self._bond_array[first:first+nbonds]['atom_index_1']
+            a2 = self._bond_array[first:first+nbonds]['atom_index_2']
+            fbonds = N.concatenate([a1[:, N.newaxis], a2[:, N.newaxis]],
+                                    axis=1)
+            for _ in range(mol['number_of_copies']):
+                bonds.append(fbonds - fatoms + atoms)
+                atoms += natoms
+            fatoms += natoms
+        return N.concatenate(bonds, axis=0)
 
 class Fragment(api.MosaicFragment):
 
@@ -248,6 +285,14 @@ class Fragment(api.MosaicFragment):
     def __repr__(self):
         return "Fragment('%s', '%s')" % (self.label, self.species)
 
+    @property
+    def parent(self):
+        ifrag = self._universe._fragment_array[self._ifrag]['parent_index']
+        if ifrag == 0:
+            return None
+        else:
+            return Fragment(self._universe, ifrag)
+
     # API implementation
     @property
     def label(self):
@@ -261,18 +306,6 @@ class Fragment(api.MosaicFragment):
             self._universe._fragment_array[self._ifrag]['species_symbol_index']
         return self._universe._symbols[isym]
 
-    @property
-    def parent(self):
-        ifrag = self._universe._fragment_array[self._ifrag]['parent_index']
-        if ifrag == 0:
-            return None
-        else:
-            return Fragment(self._universe, ifrag)
-
-    @property
-    def unique_id(self):
-        return self._ifrag
-
     @property
     def fragments(self):
         mask = self._universe._fragment_array['parent_index'] == self._ifrag
@@ -384,32 +417,28 @@ class Atom(api.MosaicAtom):
     def __repr__(self):
         return "Atom('%s')" % self.label
 
+    @property
+    def parent(self):
+        ifrag = self._universe._atom_array[self._iatom]['parent_index']
+        assert ifrag > 0
+        return Fragment(self._universe, ifrag)
+
     # API implementation
     @property
     def label(self):
         isym = self._universe._atom_array[self._iatom]['label_symbol_index']
         return self._universe._symbols[isym]
 
-    @property
-    def parent(self):
-        ifrag = self._universe._atom_array[self._iatom]['parent_index']
-        assert ifrag > 0
-        return Fragment(self._universe, ifrag)
-
-    @property
-    def unique_id(self):
-        return self._iatom
-
     @property
     def type(self):
         isym_type = \
-            self._universe._atom_array[self._iatom]['dtype_symbol_index']
+            self._universe._atom_array[self._iatom]['type_symbol_index']
         return self._universe._symbols[isym_type]
 
     @property
     def name(self):
         isym_name = \
-            self._universe._atom_array[self._iatom]['dname_symbol_index']
+            self._universe._atom_array[self._iatom]['name_symbol_index']
         return self._universe._symbols[isym_name]
 
     @property
@@ -532,6 +561,32 @@ class Configuration(api.MosaicConfiguration):
         return self._cell_parameters
 
 
+class Selection(api.MosaicSelection):
+
+    def __init__(self, universe, indices, selection_type):
+        api.validate_type(universe, Universe, "universe")
+        api.validate_array(indices, None,
+                           [N.uint8, N.uint16, N.uint32, N.uint64], "indices")
+        api.validate_value(selection_type, self._allowed_types,
+                           "selection_type")
+        self._universe = universe
+        self._indices = indices
+        self._type = selection_type
+
+    # Data Model API implementation
+    @property
+    def universe(self):
+        return self._universe
+
+    @property
+    def indices(self):
+        return self._indices
+
+    @property
+    def type(self):
+        return self._type
+
+
 class Factory(AbstractFactory):
 
     handler = MethodRegister()
@@ -557,3 +612,8 @@ class Factory(AbstractFactory):
         return Configuration(universe,
                              configuration.positions,
                              configuration.cell_parameters)
+
+    @handler(api.MosaicSelection)
+    def _make_selection(self, selection):
+        universe = self(selection.universe)
+        return Selection(universe, selection.indices, selection.type)
