@@ -1,7 +1,9 @@
 # -*- coding: utf-8 -*-
 """Immutable model
 
-This experimental module implenents the Mosaic API in terms of
+.. moduleauthor:: Konrad Hinsen
+
+This experimental module implements the Mosaic API in terms of
 immutable data structures as implemented in ImmutablePy.  The use of
 immutable data structures avoids many nasty bugs, such as modifying a
 universe after creating a configuration object that refer to it,
@@ -10,6 +12,13 @@ from ImmutablePy are not very efficient at the moment.
 
 """
 
+#-----------------------------------------------------------------------------
+#       Copyright (C) 2013 The Mosaic Development Team
+#
+#  Distributed under the terms of the BSD License.  The full license is in
+#  the file LICENSE.txt, distributed as part of this software.
+#-----------------------------------------------------------------------------
+
 import copy
 import collections
 import itertools as IT
@@ -24,6 +33,7 @@ import mosaic.api as api
 from mosaic.utility import AbstractFactory
 from mosaic.utility import MethodRegister
 from mosaic.utility import isascii
+from mosaic.utility import uint_for_max_value
 
 
 # Look up template objects in the class' instance cache.
@@ -130,185 +140,6 @@ dummy = lambda name="": _lookup(Dummy, name)
 unknown = lambda name="": _lookup(Unknown, name)
 
 
-# References to substructures
-
-class Path(ImmutableTuple):
-
-    def __repr__(self):
-        return '.'.join(self)
-
-    def __getslice__(self, start, end):
-        return Path(ImmutableTuple.__getslice__(self, start, end))
-
-    # Python 3 calls __getitem__ for slicing
-    def __getitem__(self, item):
-        if isinstance(item, slice):
-            return Path(ImmutableTuple.__getitem__(self, item))
-        else:
-            return ImmutableTuple.__getitem__(self, item)
-
-
-class StructureRef(Immutable):
-
-    def __init__(self, path):
-        assert len(path) > 0
-        self._path = path
-        self._label, self._obj = path[-1]
-
-    def __eq__(self, other):
-        return self._path == other._path
-
-    def __hash__(self):
-        return hash(self._path)
-
-    def __repr__(self):
-        return "%s(%s)" % (str(self._obj.__class__.__name__),
-                           str(self.full_path()))
-
-    def has_ancestor(self, other):
-        n = min(len(self._path), len(other._path))
-        return self._path[:n] == other._path[:n]
-
-    def toplevel_ancestor(self):
-        return _structure_ref(self._path[:1])
-
-    def ancestors(self, target=None):
-        n = len(target._path)
-        if self._path[:n] != target._path[:n]:
-            raise ValueError("%s is not an ancestor of %s"
-                             % (str(target), str(self)))
-        return [_structure_ref(self._path[:i])
-                for i in range(n + 1, len(self._path))]
-
-    def common_ancestor(self, objs):
-        paths = [obj._path for obj in objs]
-        nmax = max(len(p) for p in paths)
-        ncommon = 0
-        while ncommon < nmax:
-            if len(set(p[ncommon] for p in paths)) > 1:
-                break
-            ncommon += 1
-        return _structure_ref(paths[0][:ncommon])
-
-    def path_to(self, obj):
-        n = len(self.path)
-        if self._path != obj._path[:n]:
-            raise ValueError("%s is not an ancestor of %s"
-                             % (str(self), str(obj)))
-        return Path(a._label for a in obj._path[n:])
-
-    def full_path(self):
-        return Path(label for label, obj in self._path)
-
-    def __getitem__(self, path):
-        assert len(path) == 0
-        return self
-
-    # Data Model API implementation
-    @property
-    def label(self):
-        return self._label
-
-    @property
-    def parent(self):
-        if len(self._path) == 2:
-            return None
-        else:
-            return FragmentRef(self._path[:-1])
-
-    @property
-    def unique_id(self):
-        return '.'.join(label for label, obj in self._path[1:])
-
-
-class AtomRef(StructureRef, api.MosaicAtom):
-
-    def __init__(self, path):
-        StructureRef.__init__(self, path)
-        assert isinstance(self._obj, Atom)
-
-    # Data Model API implementation
-
-    # label and parent are implemented in StructureRef
-
-    @property
-    def name(self):
-        return self._obj.descriptor.name
-
-    @property
-    def type(self):
-        return self._obj.descriptor.type
-
-    @property
-    def number_of_sites(self):
-        return self._obj.nsites
-
-
-class FragmentRef(StructureRef, api.MosaicFragment):
-
-    def __init__(self, path):
-        StructureRef.__init__(self, path)
-        assert isinstance(self._obj, Fragment)
-
-    # Data Model API implementation
-
-    # label and parent are implemented in StructureRef
-
-    @property
-    def species(self):
-        return self._obj.species
-
-    @property
-    def fragments(self):
-        return ImmutableTuple(FragmentRef(self._path.append(f))
-                              for f in self._obj.fragments)
-
-    @property
-    def atoms(self):
-        return ImmutableTuple(AtomRef(self._path.append(a))
-                              for a in self._obj.atoms)
-
-    @property
-    def bonds(self):
-        return self._obj.bonds
-
-    @property
-    def is_polymer(self):
-        return isinstance(self._obj, Polymer)
-
-    @property
-    def polymer_type(self):
-        if isinstance(self._obj, Polymer):
-            return self._obj.polymer_type
-        else:
-            raise ValueError("polymer_type not defined in a non-polymer")
-
-    # Mapping interface
-
-    def __getitem__(self, path):
-        if isinstance(path, int) and isinstance(self._obj, Polymer):
-            return FragmentRef(self._path.append(self._obj.fragments[path]))
-        assert isascii(path) or isinstance(path, Path)
-        if isascii(path):
-            path = Path(path.split('.'))
-        if not path:
-            return self
-        child = self._obj.attrs[path[0]]
-        obj = _structure_ref(self._path.append(
-                                    ImmutableTuple((path[0], child))))
-        return obj[path[1:]]
-
-    def __len__(self):
-        return len(self._obj)
-
-
-def _structure_ref(path):
-    if isinstance(path[-1][1], Atom):
-        return AtomRef(path)
-    else:
-        return FragmentRef(path)
-
-
 # Atom and fragment templates
 
 class Atom(Immutable):
@@ -337,6 +168,30 @@ class Atom(Immutable):
     def number_of_sites(self):
         return self.nsites
 
+class AtomRef(Immutable, api.MosaicAtom):
+
+    def __init__(self, label, atom):
+        self._label = label
+        self._atom = atom
+
+    # Data Model API implementation
+
+    @property
+    def label(self):
+        return self._label
+
+    @property
+    def name(self):
+        return self._atom.descriptor.name
+
+    @property
+    def type(self):
+        return self._atom.descriptor.type
+
+    @property
+    def number_of_sites(self):
+        return self._atom.nsites
+
 atom = lambda descriptor, nsites=1: _lookup(Atom, (descriptor, nsites))
 
 
@@ -356,7 +211,7 @@ class Fragment(Immutable, collections.Mapping):
         for label, fragment in fragments:
             api.validate_label(label, "fragment label")
             if label in labels:
-                raise ValueError("labell %s occurs more than once" % label)
+                raise ValueError("label %s occurs more than once" % label)
             labels.add(label)
             api.validate_type(fragment, Fragment, "fragment template")
         self.fragments = fragments
@@ -366,7 +221,7 @@ class Fragment(Immutable, collections.Mapping):
         for label, atom in atoms:
             api.validate_label(label, "atom label")
             if label in labels:
-                raise ValueError("labell %s occurs more than once" % label)
+                raise ValueError("label %s occurs more than once" % label)
             labels.add(label)
             api.validate_type(atom, Atom, "atom")
         self.atoms = atoms
@@ -447,10 +302,6 @@ class Fragment(Immutable, collections.Mapping):
         return sum(f.number_of_bonds for l, f in self.fragments) \
                + len(self.bonds)
 
-fragment = lambda species, fragments, atoms, bonds: \
-            _lookup(Fragment, (species, fragments, atoms, bonds))
-
-
 class Polymer(Fragment, collections.Sequence):
 
     _instances = {}
@@ -470,6 +321,53 @@ class Polymer(Fragment, collections.Sequence):
         else:
             return Fragment.__getitem__(self, item)
 
+class FragmentRef(Immutable, api.MosaicFragment):
+
+    def __init__(self, label, fragment):
+        self._label = label
+        self._fragment = fragment
+
+    # Data Model API implementation
+
+    @property
+    def label(self):
+        return self._label
+
+    @property
+    def species(self):
+        return self._fragment.species
+
+    @property
+    def fragments(self):
+        return [FragmentRef(l, f) for l, f in self._fragment.fragments]
+
+    @property
+    def atoms(self):
+        return [AtomRef(l, a) for l, a in self._fragment.atoms]
+
+    @property
+    def bonds(self):
+        return self._fragment.bonds
+
+    @property
+    def is_polymer(self):
+        return isinstance(self._fragment, Polymer)
+
+    @property
+    def polymer_type(self):
+        if isinstance(self._fragment, Polymer):
+            return self._fragment.polymer_type
+        else:
+            raise ValueError("polymer_type not defined in a non-polymer")
+
+    # Sequence and mapping interface
+
+    def __getitem__(self, item):
+        return self._fragment[item]
+
+fragment = lambda species, fragments, atoms, bonds: \
+            _lookup(Fragment, (species, fragments, atoms, bonds))
+
 polymer = lambda species, fragments, bonds, polymer_type: \
             _lookup(Polymer, (species, fragments, bonds, polymer_type))
 
@@ -507,13 +405,15 @@ class Universe(Immutable, api.MosaicUniverse):
                                           and p[0].dtype == N.float64,
                                 "translation vector must be float64 "
                                 "and have shape (3,)"),))
+        if cell_shape == "infinite" \
+           and len(symmetry_transformations) > 0:
+            raise ValueError("Symmetry transformations are allowed "
+                             "only in periodic universes")
         api.validate_label(convention, "Universe.convention")
 
         self._cell_shape = cell_shape
         self._molecules = ImmutableTuple(
-                ImmutableTuple((FragmentRef(immutable((('universe', self),
-                                                       (label, fragment)))),
-                                count))
+                ImmutableTuple((FragmentRef(label, fragment), count))
                 for fragment, label, count in molecules)
         self._symmetry_transformations = symmetry_transformations
         self._convention = convention
@@ -844,6 +744,81 @@ class Configuration(Immutable, api.MosaicConfiguration):
         return self._positions
 
 
+class Selection(Immutable, api.MosaicSelection):
+
+    def __init__(self, universe, indices):
+        api.validate_type(universe, Universe, "universe")
+        self._universe = universe
+        indices = IN.array(indices, uint_for_max_value(max(indices)))
+        api.validate_indices(indices, self._max_index(), "indices")
+        self._indices = indices
+
+    def __str__(self):
+        return "%s()" % (self.__class__.__name__)
+
+    def __eq__(self, other):
+        return isinstance(other, Selection) \
+               and self.type == other.type \
+               and self.universe == other.universe \
+               and (self.indices == other.indices).all()
+
+    def __hash__(self):
+        return hash(self.type) + hash(self.universe) + hash(len(self.indices))
+
+    # Data Model API implementation
+    @property
+    def universe(self):
+        return self._universe
+
+    @property
+    def indices(self):
+        return self._indices
+
+
+class AtomSelection(Selection):
+
+    def _max_index(self):
+        return self._universe.number_of_atoms
+
+    # Data Model API implementation
+    @property
+    def type(self):
+        return 'atom'
+
+
+class SiteSelection(Selection):
+
+    def _max_index(self):
+        return self._universe.number_of_sites
+
+    # Data Model API implementation
+    @property
+    def type(self):
+        return 'site'
+
+
+class TemplateAtomSelection(Selection):
+
+    def _max_index(self):
+        return self._universe.number_of_template_atoms
+
+    # Data Model API implementation
+    @property
+    def type(self):
+        return 'template_atom'
+
+
+class TemplateSiteSelection(Selection):
+
+    def _max_index(self):
+        return self._universe.number_of_template_sites
+
+    # Data Model API implementation
+    @property
+    def type(self):
+        return 'template_site'
+
+
 class Factory(AbstractFactory):
 
     handler = MethodRegister()
@@ -897,3 +872,12 @@ class Factory(AbstractFactory):
         return Configuration(universe,
                              mosaic_configuration.positions,
                              mosaic_configuration.cell_parameters)
+
+    @handler(api.MosaicSelection)
+    def _make_selection(self, mosaic_selection):
+        universe = self(mosaic_selection.universe)
+        klass = {"atom": AtomSelection,
+                 "site": SiteSelection,
+                 "template_atom": TemplateAtomSelection,
+                 "template_site": TemplateSiteSelection}[mosaic_selection.type]
+        return klass(universe, mosaic_selection.indices)
