@@ -1,12 +1,21 @@
 # -*- coding: utf-8 -*-
 """Mutable model
 
+.. moduleauthor:: Konrad Hinsen
+
 This is the main in-memory representation of Mosaic data.  Like most
 Python code, it uses mutable data structures, which are efficient but
 can become inconsistent if not used carefully.
 
 """
 
+#-----------------------------------------------------------------------------
+#       Copyright (C) 2013 The Mosaic Development Team
+#
+#  Distributed under the terms of the BSD License.  The full license is in
+#  the file LICENSE.txt, distributed as part of this software.
+#-----------------------------------------------------------------------------
+
 import copy
 import itertools as IT
 import types
@@ -16,6 +25,7 @@ import numpy as N
 import mosaic.api as api
 from mosaic.utility import AbstractFactory
 from mosaic.utility import MethodRegister
+from mosaic.utility import uint_for_max_value
 
 
 class MosaicObject(object):
@@ -178,11 +188,6 @@ class MolecularStructureObject(MosaicObject):
     def full_path(self):
         return Path(obj._label for obj in self.ancestors())
 
-    # Data Model API implementation
-    @property
-    def unique_id(self):
-        return id(self)
-
 
 class Atom(MolecularStructureObject, api.MosaicAtom):
 
@@ -218,6 +223,10 @@ class Atom(MolecularStructureObject, api.MosaicAtom):
     def __str__(self):
         return "%s('%s')" % (self.__class__.__name__, self._label)
 
+    @property
+    def parent(self):
+        return self._parent
+
     # Data Model API implementation
     @property
     def label(self):
@@ -236,10 +245,6 @@ class Atom(MolecularStructureObject, api.MosaicAtom):
     def type(self):
         return self._descriptor.type
 
-    @property
-    def parent(self):
-        return self._parent
-
     @property
     def number_of_sites(self):
         return self.nsites
@@ -375,6 +380,10 @@ class Fragment(MolecularStructureObject, api.MosaicFragment):
             obj = obj[item]
         return obj
 
+    @property
+    def parent(self):
+        return self._parent
+
     # Data Model API implementation
     @property
     def label(self):
@@ -402,10 +411,6 @@ class Fragment(MolecularStructureObject, api.MosaicFragment):
     def polymer_type(self):
         raise ValueError("Non-polymer fragment does not have a polymer type")
 
-    @property
-    def parent(self):
-        return self._parent
-
     @property
     def fragments(self):
         return self._fragments
@@ -538,6 +543,10 @@ class Universe(MosaicObject, api.MosaicUniverse):
 
     @symmetry_transformations.setter
     def symmetry_transformations(self, symmetry_transformations):
+        if self.cell_shape == "infinite" \
+           and len(symmetry_transformations) > 0:
+            raise ValueError("Symmetry transformations are allowed "
+                             "only in periodic universes")
         api.validate_sequence(symmetry_transformations, tuple,
                               "symmetry_transformations",
                               ((lambda p: len(p) == 2, "must have length 2"),
@@ -868,6 +877,81 @@ class Configuration(MosaicObject, api.MosaicConfiguration):
         return self._positions
 
 
+class Selection(MosaicObject, api.MosaicSelection):
+
+    def __init__(self, universe, indices):
+        api.validate_type(universe, Universe, "universe")
+        self._universe = universe
+        indices = N.array(indices, uint_for_max_value(max(indices)))
+        api.validate_indices(indices, self._max_index(), "indices")
+        self._indices = indices
+
+    def __str__(self):
+        return "%s()" % (self.__class__.__name__)
+
+    def __eq__(self, other):
+        return isinstance(other, Selection) \
+               and self.type == other.type \
+               and self.universe == other.universe \
+               and (self.indices == other.indices).all()
+
+    def __hash__(self):
+        return hash(self.type) + hash(self.universe) + hash(len(self.indices))
+
+    # Data Model API implementation
+    @property
+    def universe(self):
+        return self._universe
+
+    @property
+    def indices(self):
+        return self._indices
+
+
+class AtomSelection(Selection):
+
+    def _max_index(self):
+        return self._universe.number_of_atoms
+
+    # Data Model API implementation
+    @property
+    def type(self):
+        return 'atom'
+
+
+class SiteSelection(Selection):
+
+    def _max_index(self):
+        return self._universe.number_of_sites
+
+    # Data Model API implementation
+    @property
+    def type(self):
+        return 'site'
+
+
+class TemplateAtomSelection(Selection):
+
+    def _max_index(self):
+        return self._universe.number_of_template_atoms
+
+    # Data Model API implementation
+    @property
+    def type(self):
+        return 'template_atom'
+
+
+class TemplateSiteSelection(Selection):
+
+    def _max_index(self):
+        return self._universe.number_of_template_sites
+
+    # Data Model API implementation
+    @property
+    def type(self):
+        return 'template_site'
+
+
 class Factory(AbstractFactory):
 
     handler = MethodRegister()
@@ -921,3 +1005,13 @@ class Factory(AbstractFactory):
         return Configuration(universe,
                              configuration.positions,
                              configuration.cell_parameters)
+
+    @handler(api.MosaicSelection)
+    def _make_selection(self, selection):
+        universe = self(selection.universe)
+        klass = {"atom": AtomSelection,
+                 "site": SiteSelection,
+                 "template_atom": TemplateAtomSelection,
+                 "template_site": TemplateSiteSelection}[selection.type]
+        return klass(universe, selection.indices)
+
