@@ -15,6 +15,13 @@ of the PDB data.
 
 """
 
+#-----------------------------------------------------------------------------
+#       Copyright (C) 2013 The Mosaic Development Team
+#
+#  Distributed under the terms of the BSD License.  The full license is in
+#  the file LICENSE.txt, distributed as part of this software.
+#-----------------------------------------------------------------------------
+
 from collections import OrderedDict
 import copy
 import itertools as it
@@ -28,12 +35,14 @@ from mosaic_pdb.mmcif import MMCIFParser
 
 from mosaic_pdb.space_groups import space_groups
 
-from pdb_chem_comp import components, variants
+from mosaic_pdb.pdb_chem_comp import components, variants
 from mosaic.mutable_model import \
         Element, Atom, Fragment, Polymer, Universe, \
         Configuration, SiteProperty
 from mosaic import api
 
+from mosaic.utility import isstring
+
 # Unit conversion factors
 
 class Units:
@@ -87,10 +96,11 @@ class MMCIFStructure(object):
         if pdb_code is not None:
             self.pdb_code = pdb_code
             assert structure_file is None
+            self.structure_file = None
             parser = MMCIFParser(pdb_code=self.pdb_code)
         elif structure_file is not None:
             self.pdb_code = None
-            if isinstance(structure_file, basestring):
+            if isstring(structure_file):
                 structure_file = open(structure_file)
             self.structure_file = structure_file
             parser = MMCIFParser(file_object=self.structure_file)
@@ -121,6 +131,8 @@ class MMCIFStructure(object):
                               atom_site=self.addSite,
                               atom_sites_alt=self.recordAltLabel,
                               atom_site_anisotrop=self.addAnisoU)
+        if self.structure_file is not None:
+            self.structure_file.close()
         if 'entry_id' in self.experiment:
             if self.pdb_code is None:
                 self.pdb_code = self.experiment['entry_id']
@@ -365,7 +377,6 @@ def make_monomer(sites, site_properties):
                 label_parts.append(comp_id + '_' + str(i+1))
                 fragments.append(f)
                 unique_ids.extend(u)
-                print u
             bonds = ()  # !!! TODO
             return Fragment('+'.join(label_parts), '+'.join(comp_ids),
                             fragments, (), bonds), \
@@ -595,7 +606,7 @@ def make_crystal(structure, model_number, universes):
                                      'in sequence' % seq_id)
                 residues[seq_id] = f
                 unique_ids[asym_id].extend(u)
-            residues = residues.values()
+            residues = list(residues.values())
             # Add inter-monomer links
             if polymer_type in ['polypeptide(L)']:
                 peptide_bonds = tuple((r1.get('C'), r2.get('N'), 'single')
