@@ -10,6 +10,13 @@ access that the application requires.
 
 """
 
+#-----------------------------------------------------------------------------
+#       Copyright (C) 2013 The Mosaic Development Team
+#
+#  Distributed under the terms of the BSD License.  The full license is in
+#  the file LICENSE.txt, distributed as part of this software.
+#-----------------------------------------------------------------------------
+
 # The first stage of this parser (tokenizer) was inspired by the mmCIF parser
 # in PyMMLIB (http://pymmlib.sourceforge.net/). The regular expression for
 # identifying tokens was taken directly from there. The higher-level part of
@@ -23,13 +30,22 @@ access that the application requires.
 # data is stored, and directly in the form that the application will use.
 
 import re
-import cStringIO
 import gzip
 import os
-import urllib
+import sys
 
 import numpy as np
 
+# Python 2/3 compatibility issues
+if sys.version_info[0] == 2:
+    from urllib import urlretrieve
+    bytes2text = lambda x: x
+else:
+    from urllib.request import urlretrieve
+    from io import TextIOWrapper
+    def bytes2text(stream):
+        return TextIOWrapper(stream, encoding="utf8")
+
 # Identifiers used in the return values of the parsers.
 TOKEN = 'token'
 DATA_LABEL = 'data_label'
@@ -121,7 +137,7 @@ class MMCIFParser(object):
         """
         file_iter = iter(self.file_object)
         while True:
-            line = file_iter.next()
+            line = next(file_iter)
             self.line_number += 1
 
             ## skip comments
@@ -132,7 +148,7 @@ class MMCIFParser(object):
             if line.startswith(";"):
                 lmerge = [line[1:]]
                 while True:
-                    line = file_iter.next()
+                    line = next(file_iter)
                     self.line_number += 1
                     if line.startswith(";"):
                         break
@@ -171,14 +187,14 @@ class MMCIFParser(object):
         """
         iterator = self.parseLowLevel()
         while True:
-            item_type, item = iterator.next()
+            item_type, item = next(iterator)
             if item_type is KEYWORD and item[0] == "data":
                 yield item_type, item
                 break
 
         state = LABEL_OR_KEYWORD
         while True:
-            item_type, item = iterator.next()
+            item_type, item = next(iterator)
 
             if state is LABEL_OR_KEYWORD:
                 if item_type is DATA_LABEL:
@@ -445,16 +461,16 @@ class PDBFileCollection(object):
             if self.url_pattern is None:
                 raise IOError("No URL pattern for PDB repository")
             url = self.url_pattern % pdb_code
-            filename, headers = urllib.urlretrieve(url)
-            if filename[-2:] == 'gz':
-                return gzip.GzipFile(filename)
+            filename, headers = urlretrieve(url)
+            if url.endswith('.gz'):
+                return bytes2text(gzip.GzipFile(filename))
             else:
-                return file(filename)
+                return open(filename)
         filename = self.getFilename(pdb_code)
-        if filename[-2:] == 'gz':
-            return gzip.GzipFile(filename)
+        if filename.endswith('.gz'):
+            return bytes2text(gzip.GzipFile(filename))
         else:
-            return file(filename)
+            return open(filename)
 
     def __iter__(self):
         """
