@@ -1,9 +1,13 @@
 # -*- coding: utf-8 -*-
 
 # Tests for module mosaic.hdf5
+
+#-----------------------------------------------------------------------------
+#       Copyright (C) 2013 The Mosaic Development Team
 #
-# Written by Konrad Hinsen
-#
+#  Distributed under the terms of the BSD License.  The full license is in
+#  the file LICENSE.txt, distributed as part of this software.
+#-----------------------------------------------------------------------------
 
 import copy
 import os
@@ -167,6 +171,26 @@ class PeptideTest(object):
                 self.assertTrue(l_r.is_equivalent(l))
                 self.assertTrue(is_valid(l_r))
 
+    def test_selection(self):
+        M = self.M
+        universe = self.infinite_universe
+        for s in [M.AtomSelection(universe, [0]),
+                  M.SiteSelection(universe, [1, 2]),
+                  M.TemplateAtomSelection(universe, [0, 3]),
+                  M.TemplateSiteSelection(universe, [2, 3])]:
+            self.assertTrue(is_valid(s))
+            with TestHDF5Store('w') as store:
+                self.assertRaises(IOError,
+                                  lambda: store.store("selection", s))
+                store.store("universe", universe)
+                store.store("selection", s)
+                self.assertTrue(store.retrieve("selection") is s)
+            with TestHDF5Store('r') as store:
+                s_r = store.retrieve("selection")
+            self.assertFalse(s_r is s)
+            self.assertTrue(s_r.is_equivalent(s))
+            self.assertTrue(is_valid(s_r))
+
 
 class MMPeptideTest(unittest.TestCase,
                     PeptideTest):
@@ -282,11 +306,45 @@ class IMPeptideTest(unittest.TestCase,
         TestHDF5Store.cleanup()
 
 
+class AtomicSystemTest(unittest.TestCase):
+    M = mosaic.immutable_model
+
+    def setUp(self):
+        TestHDF5Store.initialize()
+
+        M = self.M
+        C = M.element('C')
+        atom = M.fragment('carbon', (), (('C', M.atom(C)),), ())
+        self.cube_universe = M.universe('cube', ((atom, 'carbons', 20),),
+                                        convention='my_own')
+        self.assertTrue(is_valid(self.cube_universe))
+
+    def test_universe(self):
+        f = mosaic.array_model.Factory()
+        for universe in [self.cube_universe]:
+            array_universe = f(universe)
+            self.assertTrue(is_valid(array_universe))
+            with TestHDF5Store('w') as store:
+                store.store("universe", array_universe)
+                self.assertTrue(store.retrieve("universe") is array_universe)
+            with TestHDF5Store('r') as store:
+                r_universe = store.retrieve("universe")
+            self.assertFalse(r_universe is array_universe)
+            self.assertFalse(r_universe is universe)
+            self.assertTrue(r_universe.is_equivalent(array_universe))
+            self.assertTrue(r_universe.is_equivalent(universe))
+            self.assertTrue(is_valid(r_universe))
+
+    def tearDown(self):
+        TestHDF5Store.cleanup()
+
+
 def suite():
     loader = unittest.TestLoader()
     s = unittest.TestSuite()
     s.addTest(loader.loadTestsFromTestCase(MMPeptideTest))
     s.addTest(loader.loadTestsFromTestCase(IMPeptideTest))
+    s.addTest(loader.loadTestsFromTestCase(AtomicSystemTest))
     return s
 
 if __name__ == '__main__':
